/*
SonOff Mini R2 - ESP8266 - Firmware
Relay control - Contrôle du relais Version 1.0
F1ATB Avril 2023
More on F1ATB.fr
*/

#define HOSTNAME "F1ATB-SonOff-"
#define EEPROM_SIZE 200  //To strore SSID and password

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include "EEPROM.h"
#include "HTMLpages.h"

const char *ap_default_ssid;        // Access point mode IP: 192.168.4.1
const char *ap_default_psk = NULL;  // Default PSK. Free access, no password

String CLE_Rom_Init = "1234567890";
String ssid = "";
String password = "";
String DebugLines = "";

const int relay = 12;
const int led = 13;
const int inter = 4;

int bouton;
int RelayState = 0;
unsigned long startMillis;
unsigned long boutonDelay;
ESP8266WebServer server(80);

void EepromWrite(int addr, String S) {
  int L = S.length();
  for (int i = 0; i < L; ++i) {
    EEPROM.write(i + addr, S[i]);
  }
  EEPROM.write(L + addr, 0);
}
String EepromRead(int addr) {
  String S;
  while (EEPROM.read(addr) != 0) {
    S += char(EEPROM.read(addr));
    addr += 1;
  }
  return S;
}
void RelayWrite() {
  digitalWrite(relay, RelayState);
  digitalWrite(led, RelayState);
  Debug(String(millis()) + " ms -- Relay switched to :" + String(RelayState));
}
void handleRoot() {                  //Main page and Connection in Access Point Mode
  if (WiFi.getMode() != WIFI_STA) {  // in AP mode
    String NewSsid = server.arg("ssid");
    NewSsid.trim();
    String NewPassword = server.arg("passe");
    NewPassword.trim();
    Serial.println(NewSsid);
    Serial.println(NewPassword);
    if (NewSsid.length() == 0) {
      server.send(200, "text/html", String(ConnectAP_Html));
    } else {
      String S = String(SwitchWifi_Html);
      S.replace("SSID", NewSsid);
      server.send(200, "text/html", S);
      int address = 0;
      EepromWrite(address, CLE_Rom_Init);
      address += CLE_Rom_Init.length() + 1;
      EepromWrite(address, NewSsid);
      address += NewSsid.length() + 1;
      EepromWrite(address, NewPassword);
      EEPROM.commit();
      delay(5000);
      ESP.reset();
    }
  } else {  //Station Mode
    server.send(200, "text/html", String(Main_Html));
  }
}
void handleRelay() {  //Switch Relay
  String s = server.arg("R");
  RelayState = s.toInt();
  RelayWrite();
  server.send(200, "text/plain", "OK" + s);
}
void handleDebug() {  //Debug Page
  String S = "<html><head><meta http-equiv='refresh' content='3'/></head><body><code>";
  S += DebugLines + "</code><div style='margin-top:100px;width:100%;text-align:right;'><a href='/'>Main</a></div></body></html>";
  server.send(200, "text/html", S);
}

void handleNotFound() {  //Page not found
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i = 0; i < server.args(); i++) { message += " " + server.argName(i) + ": " + server.arg(i) + "\n"; }
  server.send(404, "text/plain", message);
  digitalWrite(led, 0);
}
void Debug(String S) {
  DebugLines += S + "<br> ";
  int L = DebugLines.length();
  int L0 = max(0, L - 500);
  DebugLines = DebugLines.substring(L0, L - 1);
}


void setup(void) {
  startMillis = millis();
  boutonDelay = millis();
  Serial.begin(115200);
  delay(100);
  Serial.println("\r\n");

  EEPROM.begin(512);  //To store SSID and Password

  pinMode(inter, INPUT_PULLUP);  //Switch input
  pinMode(led, OUTPUT);
  pinMode(relay, OUTPUT);
  digitalWrite(led, 0);
  digitalWrite(relay, 0);
  bouton = digitalRead(inter);
  // Set Hostname.
  String hostname(HOSTNAME);
  hostname += String(ESP.getChipId(), HEX);  //Add chip ID to hostname
  WiFi.hostname(hostname);
  Serial.println(hostname);
  Debug(hostname);
  ap_default_ssid = (const char *)hostname.c_str();
  // Check WiFi connection
  // ... check mode
  if (WiFi.getMode() != WIFI_STA) {
    WiFi.mode(WIFI_STA);
    delay(10);
  }
  //Read SSID in EEPROM
  int address = 0;
  if (EepromRead(address) == CLE_Rom_Init) {  // An SSID already stored
    address += CLE_Rom_Init.length() + 1;
    ssid = EepromRead(address);
    address += ssid.length() + 1;
    password = EepromRead(address);
  }

  if (ssid.length() > 0) {
    Debug("Wifi Begin : " + ssid);
    char namebuf[ssid.length() + 1];
    ssid.toCharArray(namebuf, ssid.length() + 1);
    char passbuf[password.length() + 1];
    password.toCharArray(passbuf, password.length() + 1);
    WiFi.begin(namebuf, passbuf);
    while (WiFi.status() != WL_CONNECTED && (millis() - startMillis < 15000)) {
      Serial.write('.');
      Serial.print(WiFi.status());
      Debug("Connection to " + ssid + " in progress...");
      delay(500);
    }
  }
  if (WiFi.status() == WL_CONNECTED) {
    // ... print IP Address
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());
    Debug("Connected IP address: " + WiFi.localIP().toString() + " or <a href='http://" + hostname + ".local' >" + hostname + ".local</a>");
  } else {
    Serial.println("Can not connect to WiFi station. Go into AP mode.");
    Debug("Cannot connect to WiFi station. Switch AP mode.");
    // Go into software AP mode.
    WiFi.mode(WIFI_AP);
    delay(10);
    WiFi.softAP(ap_default_ssid, ap_default_psk);
    Serial.print("IP address: ");
    Serial.println(WiFi.softAPIP());
    Debug("Access Point Mode. IP address: " + WiFi.softAPIP().toString());
  }
  //List of all webpages
  server.on("/", handleRoot);
  server.on("/Relay", handleRelay);
  server.on("/Debug", handleDebug);
  server.onNotFound(handleNotFound);
  server.begin();


  ArduinoOTA.setHostname((const char *)hostname.c_str());
  ArduinoOTA.begin();  //Mandatory
}

void loop(void) {
  server.handleClient();
  MDNS.update();
  ArduinoOTA.handle();                                                    //Mandatory
  if ((millis() - startMillis) > 180000 && WiFi.getMode() != WIFI_STA) {  //Connected in Access Point since 3mn. It's not normal
    Debug("Non connected in Wifi Station Mode. Restart / Pas connecté en WiFi mode Station. Redémarrage");
    delay(5000);
    ESP.restart();
  }
  if ((millis() - boutonDelay) > 200) {  //avoid glitch
    boutonDelay = millis();
    int new_bouton = digitalRead(inter);
    if (new_bouton != bouton) {
      RelayState = (RelayState + 1) % 2;
      RelayWrite();
      bouton = new_bouton;
    }
  }
}
